==> ooo_dev_tools-0.18.2-py3-none-any.whl: ooodev/calc/calc_forms.py <==
from __future__ import annotations
from typing import overload, TYPE_CHECKING
import uno
from com.sun.star.form import XForm

from ooodev.adapter.form.forms_comp import FormsComp
from ooodev.exceptions import ex as mEx
from ooodev.utils import lo as mLo
from ooodev.utils.partial.qi_partial import QiPartial
from .calc_form import CalcForm

if TYPE_CHECKING:
    from com.sun.star.form import XForms
    from .spreadsheet_draw_page import SpreadsheetDrawPage


class CalcForms(FormsComp, QiPartial):
    """
    Class for managing Calc Forms.

    This class is Enumerable and returns ``CalcForm`` instance on iteration.

    .. code-block:: python

        for sheet in doc.sheets:
            sheet["A1"].set_val("test")
            assert sheet["A1"].get_val() == "test"

    This class also as index access and returns ``CalcSheet`` instance.

    .. code-block:: python

        sheet = doc.sheets["Sheet1"]
        # or set the value of cell A2 to TEST
        doc.sheets[0]["A2"].set_val("TEST")

        # get the last sheet of the document
        last_sheet = doc.sheets[-1]

        # get the second last sheet of the document
        second_last_sheet = doc.sheets[-2]

        # get the number of sheets
        num_sheets = len(doc.sheets)

    .. versionadded:: 0.18.2
    """

    def __init__(self, owner: SpreadsheetDrawPage, forms: XForms) -> None:
        """
        Constructor

        Args:
            owner (SpreadsheetDrawPage): Owner Component
            forms (XForms): Forms instance.
        """
        self.__owner = owner
        FormsComp.__init__(self, forms)  # type: ignore
        QiPartial.__init__(self, component=forms, lo_inst=mLo.Lo.current_lo)

    def __next__(self) -> CalcForm:
        return CalcForm(owner=self, component=super().__next__())

    def __getitem__(self, index: str | int) -> CalcForm:
        if isinstance(index, int):
            if index < 0:
                index = len(self) + index
                if index < 0:
                    raise IndexError("list index out of range")
            return self.get_by_index(index)
        return self.get_by_name(index)

    def __len__(self) -> int:
        return self.component.getCount()

    def _get_index(self, idx: int, allow_greater: bool = False) -> int:
        """
        Gets the index.

        Args:
            idx (int): Index of sheet. Can be a negative value to index from the end of the list.
            allow_greater (bool, optional): If True and index is greater then the number of
                sheets then the index becomes the next index if sheet were appended. Defaults to False.

        Returns:
            int: Index value.
        """
        count = len(self)
        if idx < 0:
            if count == 0 and allow_greater:
                return 0
            idx = count + idx
            if idx < 0:
                raise IndexError("list index out of range")
        if idx >= count:
            if allow_greater:
                idx = count
            else:
                raise IndexError("list index out of range")
        return idx

    def _create_name(self, name: str) -> str:
        used_name = True
        i = 1
        nm = f"{name}{i}"
        while used_name:
            used_name = self.has_by_name(nm)
            if used_name:
                i += 1
                nm = f"{name}{i}"
        return nm

    # region add_form
    @overload
    def add_form(self) -> CalcForm:
        """
        Adds a new form at the end.

        Returns:
            CalcForm: Form
        """
        ...

    @overload
    def add_form(self, idx: int) -> CalcForm:
        """
        Adds a new form.

        Args:
            idx (int): Index of form.

        Returns:
            CalcForm: Form
        """
        ...

    @overload
    def add_form(self, name: str) -> CalcForm:
        """
        Adds a new form.

        Args:
            name (str): Name of form.

        Returns:
            CalcForm: Form
        """
        ...

    def add_form(self, *args, **kwargs) -> CalcForm:
        """
        Adds a new form.

        Args:
            name (str): Name of form.
            idx (int): Index of form.

        Raises:
            NameClashError: If name already exists.

        Returns:
            CalcForm: Form
        """
        all_args = [arg for arg in args]
        all_args.extend(kwargs.values())
        if len(all_args) == 0:
            # add a new form at the end
            all_args.append(-1)

        if len(all_args) != 1:
            raise TypeError("add_form() takes 1 argument but {} were given".format(len(all_args)))
        arg1 = all_args[0]
        if isinstance(arg1, int):
            idx = self._get_index(arg1, allow_greater=True)
            frm = mLo.Lo.create_instance_mcf(XForm, "stardiv.one.form.component.Form", raise_err=True)
            frm.Name = self._create_name("Form")  # type: ignore
            self.insert_by_index(idx, frm)
            return self.get_by_index(idx)
        elif isinstance(arg1, str):
            if self.has_by_name(arg1):
                raise mEx.NameClashError(f"Name '{arg1}' already exists")
            frm = mLo.Lo.create_instance_mcf(XForm, "stardiv.one.form.component.Form", raise_err=True)
            self.insert_by_name(arg1, frm)
            return self.get_by_name(arg1)
        else:
            raise TypeError("add_form() argument must be int or str")

    # endregion add_form

    # region XIndexAccess overrides

    def get_by_index(self, idx: int) -> CalcForm:
        """
        Gets the element at the specified index.

        Args:
            idx (int): The Zero-based index of the element. Idx can be a negative value to index from the end of the list.
                For example, -1 will return the last element.

        Returns:
            CalcForm: The element at the specified index.
        """
        if idx < 0:
            idx = len(self) + idx
            if idx < 0:
                raise IndexError("Index out of range")
        if idx >= len(self):
            raise IndexError(f"Index out of range: '{idx}'")

        result = super().get_by_index(idx)
        return CalcForm(owner=self, component=result)

    # endregion XIndexAccess overrides

    # region XNameAccess overrides

    def get_by_name(self, name: str) -> CalcForm:
        """
        Gets the element with the specified name.

        Args:
            name (str): The name of the element.

        Raises:
            MissingNameError: If sheet is not found.

        Returns:
            CalcForm: The element with the specified name.
        """
        if not self.has_by_name(name):
            raise mEx.MissingNameError(f"Unable to find sheet with name '{name}'")
        result = super().get_by_name(name)
        return CalcForm(owner=self, component=result)

    # endregion XNameAccess overrides

    # region Properties
    @property
    def owner(self) -> SpreadsheetDrawPage:
        """
        Returns:
            SpreadsheetDrawPage: Calc doc
        """
        return self.__owner

    # endregion Properties

==> ooo_dev_tools-0.25.1-py3-none-any.whl: ooodev/calc/calc_forms.py <==
from __future__ import annotations
from typing import overload, TYPE_CHECKING
import uno
from com.sun.star.form import XForm

from ooodev.adapter.form.forms_comp import FormsComp
from ooodev.exceptions import ex as mEx
from ooodev.utils import gen_util as mGenUtil
from ooodev.utils import lo as mLo
from ooodev.utils.inst.lo.lo_inst import LoInst
from ooodev.utils.partial.lo_inst_props_partial import LoInstPropsPartial
from ooodev.utils.partial.qi_partial import QiPartial
from ooodev.utils.partial.service_partial import ServicePartial
from .calc_form import CalcForm

if TYPE_CHECKING:
    from com.sun.star.form import XForms
    from .spreadsheet_draw_page import SpreadsheetDrawPage


class CalcForms(LoInstPropsPartial, FormsComp, ServicePartial, QiPartial):
    """
    Class for managing Calc Forms.

    This class is Enumerable and returns ``CalcForm`` instance on iteration.

    .. code-block:: python

        for sheet in doc.sheets:
            sheet["A1"].set_val("test")
            assert sheet["A1"].get_val() == "test"

    This class also as index access and returns ``CalcSheet`` instance.

    .. code-block:: python

        sheet = doc.sheets["Sheet1"]
        # or set the value of cell A2 to TEST
        doc.sheets[0]["A2"].set_val("TEST")

        # get the last sheet of the document
        last_sheet = doc.sheets[-1]

        # get the second last sheet of the document
        second_last_sheet = doc.sheets[-2]

        # get the number of sheets
        num_sheets = len(doc.sheets)

    .. versionadded:: 0.18.2
    """

    def __init__(self, owner: SpreadsheetDrawPage, forms: XForms, lo_inst: LoInst | None = None) -> None:
        """
        Constructor

        Args:
            owner (SpreadsheetDrawPage): Owner Component
            forms (XForms): Forms instance.
        """
        if lo_inst is None:
            lo_inst = mLo.Lo.current_lo
        self._owner = owner
        LoInstPropsPartial.__init__(self, lo_inst=lo_inst)
        FormsComp.__init__(self, forms)  # type: ignore
        ServicePartial.__init__(self, component=forms, lo_inst=self.lo_inst)
        QiPartial.__init__(self, component=forms, lo_inst=self.lo_inst)

    def __next__(self) -> CalcForm:
        return CalcForm(owner=self, component=super().__next__(), lo_inst=self.lo_inst)

    def __getitem__(self, index: str | int) -> CalcForm:
        if isinstance(index, int):
            return self.get_by_index(index)
        return self.get_by_name(index)

    def __len__(self) -> int:
        return self.component.getCount()

    def _get_index(self, idx: int, allow_greater: bool = False) -> int:
        """
        Gets the index.

        Args:
            idx (int): Index of sheet. Can be a negative value to index from the end of the list.
            allow_greater (bool, optional): If True and index is greater then the number of
                sheets then the index becomes the next index if sheet were appended. Defaults to False.

        Returns:
            int: Index value.
        """
        count = len(self)
        return mGenUtil.Util.get_index(idx, count, allow_greater)

    def _create_name(self, name: str) -> str:
        used_name = True
        i = 1
        nm = f"{name}{i}"
        while used_name:
            used_name = self.has_by_name(nm)
            if used_name:
                i += 1
                nm = f"{name}{i}"
        return nm

    # region add_form
    @overload
    def add_form(self) -> CalcForm:
        """
        Adds a new form at the end.

        Returns:
            CalcForm: Form
        """
        ...

    @overload
    def add_form(self, idx: int) -> CalcForm:
        """
        Adds a new form.

        Args:
            idx (int): Index of form.

        Returns:
            CalcForm: Form
        """
        ...

    @overload
    def add_form(self, name: str) -> CalcForm:
        """
        Adds a new form.

        Args:
            name (str): Name of form.

        Returns:
            CalcForm: Form
        """
        ...

    def add_form(self, *args, **kwargs) -> CalcForm:
        """
        Adds a new form.

        Args:
            name (str): Name of form.
            idx (int): Index of form.

        Raises:
            NameClashError: If name already exists.

        Returns:
            CalcForm: Form
        """
        all_args = [arg for arg in args]
        all_args.extend(kwargs.values())
        if len(all_args) == 0:
            # add a new form at the end
            all_args.append(-1)

        if len(all_args) != 1:
            raise TypeError("add_form() takes 1 argument but {} were given".format(len(all_args)))
        arg1 = all_args[0]
        if isinstance(arg1, int):
            idx = self._get_index(arg1, allow_greater=True)
            frm = self.lo_inst.create_instance_mcf(XForm, "stardiv.one.form.component.Form", raise_err=True)
            frm.Name = self._create_name("Form")  # type: ignore
            self.insert_by_index(idx, frm)
            return self.get_by_index(idx)
        elif isinstance(arg1, str):
            if self.has_by_name(arg1):
                raise mEx.NameClashError(f"Name '{arg1}' already exists")
            frm = self.lo_inst.create_instance_mcf(XForm, "stardiv.one.form.component.Form", raise_err=True)
            self.insert_by_name(arg1, frm)
            return self.get_by_name(arg1)
        else:
            raise TypeError("add_form() argument must be int or str")

    # endregion add_form

    # region XIndexAccess overrides

    def get_by_index(self, idx: int) -> CalcForm:
        """
        Gets the element at the specified index.

        Args:
            idx (int): The Zero-based index of the element. Idx can be a negative value to index from the end of the list.
                For example, -1 will return the last element.

        Returns:
            CalcForm: The element at the specified index.
        """
        idx = self._get_index(idx, True)
        result = super().get_by_index(idx)
        return CalcForm(owner=self, component=result, lo_inst=self.lo_inst)

    # endregion XIndexAccess overrides

    # region XNameAccess overrides

    def get_by_name(self, name: str) -> CalcForm:
        """
        Gets the element with the specified name.

        Args:
            name (str): The name of the element.

        Raises:
            MissingNameError: If sheet is not found.

        Returns:
            CalcForm: The element with the specified name.
        """
        if not self.has_by_name(name):
            raise mEx.MissingNameError(f"Unable to find sheet with name '{name}'")
        result = super().get_by_name(name)
        return CalcForm(owner=self, component=result, lo_inst=self.lo_inst)

    # endregion XNameAccess overrides

    # region Properties
    @property
    def owner(self) -> SpreadsheetDrawPage:
        """
        Returns:
            SpreadsheetDrawPage: Calc doc
        """
        return self._owner

    # endregion Properties
